﻿using System;
using System.Collections.Generic;
using Ozeki.MediaGateway;
using Ozeki.VoIP;

namespace Web2SIPServer
{
    class SIPGateway : MediaGateway
    {
        Dictionary<IClient, MyClient> _myClients;
        ISoftPhone _softPhone;
        SIPAccountPool _sipAccountPool;
        IStreamService _streamService;

        /// <summary>
        /// Creates a new instance of SIPGateway.
        /// </summary>
        public SIPGateway()
        {
            Console.WriteLine("SIP Gateway starting...");
        }

        /// <summary>
        /// Indicates when the mediaGateway successfully started and this time It initialize softphone and other things.
        /// </summary>
        public override void OnStart()
        {
            Console.WriteLine("SIP Gateway started");

            _myClients = new Dictionary<IClient, MyClient>();
            _softPhone = SoftPhoneFactory.CreateSoftPhone(20000, 20500);

            _sipAccountPool = new SIPAccountPool();

            for (int i = 50; i < 59; i++)
            {
                var name = "8" + i;
                var sipAccount = new SIPAccount(true, name, name, name, name, "192.168.115.100", 5060);

                _sipAccountPool.AddSIPAcount(sipAccount);
            }

            _streamService = GetService<IStreamService>();
        }

        /// <summary>
        /// Indicates the client connection and registers it if the chatclient dictionary doesn't contain the client nickname.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="parameters">Opional parameters</param>
        public override void OnClientConnect(IClient client, object[] parameters)
        {
            Console.WriteLine("New client connected. Remote address: {0}", client.RemoteAddress);

            var sipAccount = _sipAccountPool.GetSIPAccount();

            if (sipAccount == null)
            {
                client.InvokeMethod("OnRegistrationStateChanged", "Connection limit");
                client.Close();
                return;
            }

            var myClient = new MyClient(client, _streamService, _softPhone, sipAccount);
            _myClients.Add(client, myClient);

            myClient.Register();
        }

        /// <summary>
        ///  Indicates client disconnection and remove client's keyvaluepair from chatclient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public override void OnClientDisconnect(IClient client)
        {
            Console.WriteLine("Client disconnected");

            lock (_myClients)
            {
                MyClient myClient;
                if (_myClients.TryGetValue(client, out myClient))
                {
                    myClient.Unregister();
                    _sipAccountPool.AddSIPAcount(myClient.Account);
                    _myClients.Remove(client);
                }
            }

        }

        /// <summary>
        /// Indicates the client starts publish his IMediastream.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="mediaStream">Published IMediaStream</param>
        public override void OnStreamPublishStart(IClient client, IMediaStream mediaStream)
        {
            Console.WriteLine("Media stream published. Stream name {0}", mediaStream.Name);

            lock (_myClients)
            {
                MyClient myClient;
                if (_myClients.TryGetValue(client, out myClient))
                    myClient.StreamPublished(mediaStream);
            }

        }

        /// <summary>
        /// Indicates the client close his IMediaStream.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="mediaStream">Closed IMediaStream</param>
        public override void OnStreamClose(IClient client, IMediaStream mediaStream)
        {
            Console.WriteLine("Media stream closed.");

            lock (_myClients)
            {
                MyClient myClient;
                if (_myClients.TryGetValue(client, out myClient))
                    myClient.StreamClosed();
            }

        }

        /// <summary>
        /// Forward the call request to the MyClient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="phoneNumber">dial number</param>
        public void Call(IClient client, string phoneNumber)
        {
            lock (_myClients)
            {
                MyClient myClient;
                if (_myClients.TryGetValue(client, out myClient))
                    myClient.Call(phoneNumber);
            }

        }


        /// <summary>
        /// Forward the hangup request to the MyClient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void HangUp(IClient client)
        {
            lock (_myClients)
            {
                MyClient myClient;
                if (_myClients.TryGetValue(client, out myClient))
                    myClient.HangUp();
            }

        }

        /// <summary>
        /// Forwards the Accept request to the MyClient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void Accept(IClient client)
        {
            lock (_myClients)
            {
                MyClient myClient;
                if (_myClients.TryGetValue(client, out myClient))
                    myClient.Accept();
            }

        }

        /// <summary>
        /// Reject an incoming call request.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void Reject(IClient client)
        {
            lock (_myClients)
            {
                MyClient myClient;
                if (_myClients.TryGetValue(client, out  myClient))
                    myClient.Reject();
            }

        }
    }
}
